﻿using UnityEngine;
using System.Collections;

public class BotWanderer : MonoBehaviour 
{
    [SerializeField] float wanderDistance = 4.0f;
    [SerializeField] float idleTime = 1.0f;
    [SerializeField] float wanderDuration = 1.0f;
    [SerializeField] LayerMask pathTestMask;

    private Vector3 direction = Vector3.zero;
    private float nextWander;

    private float acceleration;

	void Start () 
    {
        nextWander = Time.time + Random.Range(0.0f, wanderDuration + idleTime);
        GetNewDirection();

        acceleration = 4 * wanderDistance / (wanderDuration*wanderDuration);
	}
	
	void FixedUpdate () 
    {
        if (Time.time > nextWander)
        {
            float wanderPercentage = (Time.time - nextWander) / wanderDuration;

            if (wanderPercentage > 1.0f)
            {
                nextWander = Time.time + idleTime;
                GetNewDirection();
            }
            else
            {
                if (wanderPercentage < .5f)
                {
                    rigidbody.AddForce(direction * acceleration, ForceMode.Acceleration);
                }
                else
                {
                    rigidbody.AddForce(-direction * acceleration, ForceMode.Acceleration);
                }
            }
        }
        else
        {
            //Idling, counteract velocity
            Vector3 motionDirection = rigidbody.velocity.normalized;
            rigidbody.AddForce(-motionDirection * acceleration, ForceMode.Acceleration);
        }
	}

    private void GetNewDirection()
    {
        int maxTests = 5;
        int testCount = 0;
        
        RaycastHit hit;
        do
        {
            Vector2 randomDirection = Random.insideUnitCircle.normalized;
            direction = new Vector3(randomDirection.x, randomDirection.y, 0);
            testCount++;
        } while (Physics.SphereCast(transform.position, 1.0f, direction, out hit, wanderDistance, pathTestMask) && testCount < maxTests);
    }
}
