﻿using UnityEngine;
using System.Collections;

public class DestroyOnImpact : MonoBehaviour 
{
    [SerializeField] GameObject destructionEffect = null;
    [SerializeField] float maxImpactStrength = 50.0f;

    void OnCollisionEnter(Collision hit)
    {
        //Determine impact strength
        float dotImpact = Vector3.Dot(hit.contacts[0].normal, hit.relativeVelocity);
        float strength = dotImpact * (hit.rigidbody ? hit.rigidbody.mass : rigidbody.mass);

        if (strength > maxImpactStrength)
        {
            if (hit.rigidbody)
            {
                hit.rigidbody.SendMessage("MESSAGE_BotImpact", SendMessageOptions.DontRequireReceiver);
            }

            if (destructionEffect)
            {
                Instantiate(destructionEffect, hit.contacts[0].point, Quaternion.identity);
            }

            BroadcastMessage("MESSAGE_Destroy", SendMessageOptions.DontRequireReceiver);
            GameManager.Instance.RegisterKill(gameObject);

            Destroy(gameObject);
        }
    }
}
