﻿using UnityEngine;
using System.Collections;

public class PauseAndShoot : MonoBehaviour 
{
    [SerializeField] MonoBehaviour[] toToggle;
    [SerializeField] GameObject[] toSetActive;
    [SerializeField] GameObject projectilePrefab;
    [SerializeField] GameObject muzzleEffect;

    [SerializeField] float projectileSpeed = 10.0f;
    [SerializeField] float pauseTimeBeforeFiring = .5f;
    [SerializeField] float shotRefireDelay = .3f;
    [SerializeField] float projectileSpread = 5.0f;
    [SerializeField] Transform muzzleTransform;
    [SerializeField] int shotsPerBurst = 4;
    [SerializeField] float minPauseInterval = 5.0f;
    [SerializeField] float maxPauseInterval = 10.0f;

    [SerializeField] AudioClip shotClip;

    void Start()
    {
        StartCoroutine(FireOnTarget()); 
    }

    private IEnumerator FireOnTarget()
    {
        while (true)
        {
            // Wait for interval
            yield return new WaitForSeconds(Random.Range(minPauseInterval, maxPauseInterval));

            // Pause, fire burst!
            foreach (MonoBehaviour b in toToggle)
            {
                b.enabled = !b.enabled;
            }
            foreach (GameObject g in toSetActive)
            {
                g.SetActive(true);
            }

            yield return new WaitForSeconds(pauseTimeBeforeFiring);

            for (int i = 0; i < shotsPerBurst; i++)
            {
                Quaternion spreadRotation = Quaternion.AngleAxis(Random.Range(-projectileSpread, projectileSpread), Vector3.forward);

                GameObject effect = Instantiate(muzzleEffect, muzzleTransform.position, muzzleTransform.rotation) as GameObject;
                effect.transform.parent = muzzleTransform;

                GameObject projectile = Instantiate(projectilePrefab, muzzleTransform.position, spreadRotation * muzzleTransform.rotation) as GameObject;
                projectile.rigidbody.velocity = projectile.transform.forward * projectileSpeed;

                audio.PlayOneShot(shotClip, .5f);

                yield return new WaitForSeconds(shotRefireDelay);
            }

            // Resume!
            foreach (MonoBehaviour b in toToggle)
            {
                b.enabled = !b.enabled;
            }
            foreach (GameObject g in toSetActive)
            {
                g.SetActive(false);
            }
        }
    }
}
