﻿using UnityEngine;
using System.Collections;

public class SpawnInRadius : MonoBehaviour 
{
    [SerializeField] Transform origin;
    [SerializeField] float originSpawnRange = 2.0f;
    [SerializeField] float spawnRange = 1.0f;
    [SerializeField] float spawnInterval = 5.0f;
    [SerializeField] float fieldEnterDuration = 2.0f;
    [SerializeField] AnimationCurve curveX = AnimationCurve.EaseInOut(0,0,1,1);
    [SerializeField] AnimationCurve curveY = AnimationCurve.EaseInOut(0,0,1,1);
    [SerializeField] AnimationCurve curveZ = AnimationCurve.EaseInOut(0,0,1,1);

    private GameObject lastObject;
    private float lastObjectCreationTime;
    private Vector3 lastObjectCreationPosition;
    private Vector3 lastObjectTarget;
    private Vector3 lastObjectVelocity;

    public bool AvailableForSpawn
    {
        get { return lastObject == null; }
    }

    void Update()
    {
        if (lastObject)
        {
            float timeAlong = (Time.time - lastObjectCreationTime) / fieldEnterDuration;

            Vector3 newPosition;
            newPosition.x = Mathf.Lerp(lastObjectCreationPosition.x, lastObjectTarget.x, curveX.Evaluate(timeAlong));
            newPosition.y = Mathf.Lerp(lastObjectCreationPosition.y, lastObjectTarget.y, curveY.Evaluate(timeAlong));
            newPosition.z = Mathf.Lerp(lastObjectCreationPosition.z, lastObjectTarget.z, curveZ.Evaluate(timeAlong));

            lastObject.rigidbody.MovePosition(newPosition);

            if (timeAlong > 1.0f)
            {
                Vector3 pos = lastObject.transform.position;
                pos.z = 0;
                lastObject.transform.position = pos;
                lastObject.rigidbody.isKinematic = false;

                lastObject = null;                
            }
        }
    }

    public GameObject SpawnObject(GameObject sourceObject)
    {
        if (!AvailableForSpawn)
            return null;

        Vector2 spawnLocation = Random.insideUnitCircle * spawnRange;

        lastObject = Instantiate(sourceObject, origin.position + Random.insideUnitSphere * originSpawnRange, Quaternion.identity) as GameObject;
        lastObjectCreationPosition = lastObject.transform.position;
        lastObjectTarget = transform.position + new Vector3(spawnLocation.x,spawnLocation.y,0);
        lastObjectCreationTime = Time.time;
        lastObject.rigidbody.isKinematic = true;

        return lastObject;
    }

    void OnDrawGizmosSelected()
    {
        Gizmos.DrawWireSphere(transform.position, spawnRange);
        Gizmos.DrawWireSphere(origin.position, originSpawnRange);
    }
}
