﻿using UnityEngine;
using System.Linq;
using System.Collections;
using System.Collections.Generic;

public class WaveManager : MonoBehaviour
{

    [SerializeField] GameObject basicBotPrefab;
    [SerializeField] GameObject advancedBotPrefab;
    [SerializeField] SpawnInRadius[] basicBotSpawners;
    [SerializeField] float waveSpawnSpeed = 0.2f; //TODO: Throttle by wave number
    [SerializeField] float waveDelay = 2.0f;
    [SerializeField] float waveInnerDelay = 3.0f;
    [SerializeField] int firstWaveCount = 3;
    [SerializeField] int waveEscalation = 3;
    [SerializeField] int enemiesPerAdvancedBot = 8;
    [SerializeField] int emergencyBotSpawnCount = 3;

    private int waveEnemyCount = 0;

    List<GameObject> basicBots = new List<GameObject>();
    List<GameObject> advancedBots = new List<GameObject>();

    // Use this for initialization
    void Start() 
    {
        waveEnemyCount = firstWaveCount;
        StartCoroutine(WaveSpawner());
	}

    void Update()
    {
        basicBots.RemoveAll(x => x == null);
        advancedBots.RemoveAll(x => x == null);
    }

    IEnumerator WaveSpawner()
    {
        while(GameManager.Instance.PlayerAlive)
        {
            int advancedBotCount = waveEnemyCount / enemiesPerAdvancedBot;

            int subWaveCount = -waveEnemyCount/3;
            for (int i = 0; i < waveEnemyCount; i++)
            {
                subWaveCount++;

                if (subWaveCount >= waveEscalation)
                {
                    subWaveCount = 0;
                    yield return new WaitForSeconds(waveInnerDelay);
                }

                SpawnInRadius[] availableSpawns;

                do
                {
                    yield return new WaitForSeconds(waveSpawnSpeed);
                    availableSpawns = basicBotSpawners.Where(x => x.AvailableForSpawn).ToArray();
                }
                while(availableSpawns.Length == 0);


                if (i < advancedBotCount) // Spawn advanced bot!
                {
                    GameObject enemy = availableSpawns[Random.Range(0, availableSpawns.Length)].SpawnObject(advancedBotPrefab);
                    if (enemy)
                        advancedBots.Add(enemy);
                }
                else
                {
                    GameObject enemy = availableSpawns[Random.Range(0, availableSpawns.Length)].SpawnObject(basicBotPrefab);
                    if (enemy)
                        basicBots.Add(enemy);
                }
            }

            while (basicBots.Count + advancedBots.Count > 0)
            {
                if (advancedBots.Count > 0 && basicBots.Count == 0)
                {
                    // Spawn a few basic bots to provide ammunition. Remove point value
                    for (int i = 0; i < emergencyBotSpawnCount; i++)
                    {
                        SpawnInRadius[] availableSpawns;

                        do
                        {
                            yield return new WaitForSeconds(waveSpawnSpeed);
                            availableSpawns = basicBotSpawners.Where(x => x.AvailableForSpawn).ToArray();
                        }
                        while (availableSpawns.Length == 0);

                        GameObject enemy = availableSpawns[Random.Range(0, availableSpawns.Length)].SpawnObject(basicBotPrefab);
                        PointValue pv = enemy.GetComponent<PointValue>();
                        pv.RemovePointValue();
                        if (enemy)
                            basicBots.Add(enemy);
                    }
                }

                yield return null;
            }

            // Wave full spawned: escalate!
            waveEnemyCount += waveEscalation;
            yield return new WaitForSeconds(waveDelay);
        }
    }
}
