﻿using UnityEngine;
using System.Collections;

public class GameManager : MonoBehaviour 
{

    private int score = 0;
    private bool playerAlive = true;

    private static GameManager instance;
    public static GameManager Instance
    {
        get
        {
            if (instance)
                return instance;

            GameObject instanceObject = new GameObject("GameManager");
            instance = instanceObject.AddComponent<GameManager>();

            return instance;
        }
    }

    public int Score
    {
        get { return score; }
    }

    public bool PlayerAlive
    {
        get { return playerAlive; }
    }

    void OnGUI()
    {
        if (playerAlive)
        {
            Rect scoreRect = new Rect(Screen.width * .44f, 10.0f, Screen.width * .12f, 40.0f);
            Texture2D backingTexture = new Texture2D(1, 1);
            backingTexture.Apply();
            backingTexture.SetPixel(0, 0, new Color(0.0f, 0.0f, 0.0f, 0.3f));

            GUI.Box(scoreRect, backingTexture);

            GUIStyle scoreStyle = new GUIStyle();
            scoreStyle.alignment = TextAnchor.MiddleCenter;
            scoreStyle.fontSize = 36;
            scoreStyle.normal.textColor = new Color(0.0f, 0.9f, 1.0f, 0.85f);

            GUILayout.BeginArea(scoreRect);
            GUILayout.Label("" + Score, scoreStyle);
            GUILayout.EndArea();
        }

        if (!playerAlive)
        {
            GUIStyle titleStyle = new GUIStyle();
            titleStyle.alignment = TextAnchor.MiddleCenter;
            titleStyle.fontSize = 32;
            titleStyle.normal.textColor = Color.white;

            GUIStyle messageStyle = new GUIStyle();
            messageStyle.alignment = TextAnchor.MiddleCenter;
            messageStyle.normal.textColor = Color.white;
            

            GUILayout.BeginArea(new Rect(Screen.width * .2f, Screen.height * .4f, Screen.width * .6f, Screen.height * .2f));
            GUILayout.Label("GAME OVER!", titleStyle);
            GUILayout.Label("Final score was " + score, messageStyle);
            GUILayout.BeginHorizontal();
            GUILayout.FlexibleSpace();
            if (GUILayout.Button("Return to main menu"))
            {
                Application.LoadLevel("MenuScene");
            }
            if (GUILayout.Button("(R) Reset Stage") || Input.GetKeyDown(KeyCode.R))
            {
                Application.LoadLevel(Application.loadedLevel);
            }
            GUILayout.FlexibleSpace();
            GUILayout.EndHorizontal();
            GUILayout.EndArea();
        }
    }

    public void InitializeRound()
    {
    }

    public void RegisterEnemy(GameObject obj)
    {
    }

    private float comboWindow = 2.0f;
    private float comboStart = 0;
    private int killCount = 0;
    private int comboBonus = 5;
    private int chainBonus = 5;

    public void RegisterKill(GameObject obj)
    {
        if (Time.time - comboStart < comboWindow)
        {
            killCount++;
        }
        else
        {
            killCount = 0;
        }
        comboStart = Time.time;

        int newPoints = 0;

        PointValue pv = obj.GetComponent<PointValue>();
        if (pv)
            newPoints += pv.PointsForKill;

        if (pv.PointsForKill > 0)
            newPoints += comboBonus * killCount;

        ChainWatcher watcher = obj.GetComponent<ChainWatcher>();
        if (!watcher.DirectImpact)
        {
            newPoints += chainBonus;
        }

        CreatePointNotification(obj, newPoints);
        score += newPoints;
    }

    public void RegisterDeath()
    {
        playerAlive = false;
    }

    private void CreatePointNotification(GameObject obj, int points)
    {
        GameObject notification = Instantiate(Resources.Load<GameObject>("PointNotification")) as GameObject;
        notification.transform.position = obj.transform.position;

        PointNotification pn = notification.GetComponent<PointNotification>();
        string special = "";
        if (points == 0)
            special = "CHAFF";
        if (killCount > 10)
        {
            special += " RAMPAGE";
        }
        else if (killCount > 5)
        {
            special += " STREAK";
        }
        else if (killCount > 0)
        {
            special += " COMBO";
        }

        ChainWatcher watcher = obj.GetComponent<ChainWatcher>();
        if (!watcher.DirectImpact)
        {
            special += " CHAIN";
        }

        pn.SetMessage(special, points);
    }
}
