﻿using UnityEngine;
using System.Collections;

public class PointNotification : MonoBehaviour 
{
    [SerializeField] float boundX = 5.0f;
    [SerializeField] float boundY = 5.0f;
    [SerializeField] TextMesh text;
    [SerializeField] Color startColor;
    [SerializeField] Color endColor;
    [SerializeField] float startScale;
    [SerializeField] float endScale;
    [SerializeField] Vector3 startPos;
    [SerializeField] Vector3 endPos;
    [SerializeField] AnimationCurve curve;
    [SerializeField] float animationTime;

    private float startTime = 0;

    public void SetMessage(string special, int pointValue)
    {
        text.text = special + " " + pointValue;
    }

    void Start()
    {
        startTime = Time.time;
    }

    void Update()
    {
        Vector3 pos = transform.position;
        pos.x = Mathf.Clamp(pos.x, -boundX, boundX);
        pos.y = Mathf.Clamp(pos.y, 0, boundY);
        transform.position = pos;

        float timeAlong = (Time.time - startTime) / animationTime;

        float curveVal = curve.Evaluate(timeAlong);

        text.transform.localPosition = Vector3.Lerp(startPos, endPos, curveVal);
        text.transform.localScale = Vector3.one * Mathf.Lerp(startScale, endScale, curveVal);
        text.color = Color.Lerp(startColor, endColor, curveVal);
    }
}
